package alrosine.ean.upc;

import java.io.InputStream;

/**
 * UpcQueryService provides encapsulates the retrieval of UPC information from the eandata.com website.
 */
public interface UpcQueryService {

	/**
	 * Retrieves and formats the EAN response for the desired UPC.
	 *
	 * @param upc - The UPC code for which EAN data is to be retrieved.
	 *
	 * @return UpcResponse - Formatted implementation of the available EAN data.
	 */
	public UpcResponse getUpcResponse(String upc);

	/**
	 * Retrieves and passes through the exact UPC information retrieved from the eandata.com website.
	 *
	 * @param upc - The UPC code for which EAN data is to be retrieved.
	 *
	 * @return InputStream - An InputStream that can be used to retrieve and process the EAN data directly.
	 */
	public InputStream getDataStream(String upc);

}
