package alrosine.ean.upc.impl;

import alrosine.ean.upc.UpcResponse;

import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;

/**
 * CSV (comma separated values) implementation of the UpcQueryParser interface
 */
public class UpcQueryCsvParser implements UpcQueryParser {

	private static String STATUS_ELEMENT_NAME = "status";
	private static String PRODUCT_ELEMENT_NAME = "product";
	private static String COMPANY_ELEMENT_NAME = "company";

	private boolean inStatus = false;
	private boolean inProduct = false;
	private boolean inCompany = false;

	private boolean autoRetrieveGraphics = false;
	private UpcResponseImpl response;

	public UpcQueryCsvParser() {
	}

	public UpcQueryCsvParser(boolean autoRetrieveGraphics) {
		this.autoRetrieveGraphics = autoRetrieveGraphics;
	}

	@Override
	public UpcQueryServiceMode getUpcQueryServiceMode() {
		return UpcQueryServiceMode.Csv;
	}

	@Override
	public UpcResponse getUpcResponse(InputStream inputStream) {
		try {
			int availableData = inputStream.available();
			byte[] inputData = new byte[inputStream.available()];
			inputStream.read(inputData);
			parse(inputData);
			return this.response;
		} catch(IOException e) {
			e.printStackTrace();
			throw new RuntimeException("Attempt to parse InputStream failed.", e);
		}
	}

	private void parse(byte[] inputData) {
		this.response = new UpcResponseImpl();
		UpcProductImpl product = new UpcProductImpl();
		UpcCompanyImpl company = new UpcCompanyImpl();
		this.response.setProduct(product);
		this.response.setCompany(company);
		String stringData = new String(inputData);
		stringData = stringData.replace("\n", ",");
		String[] tokens = stringData.split(",");
		int idx = 0;
		for(String token : tokens) {
			token = token.replace("\"", "");
			if (STATUS_ELEMENT_NAME.equalsIgnoreCase(token)) {
				inStatus = true;
				inProduct = false;
				inCompany = false;
				idx = 0;
				continue;
			} else if (PRODUCT_ELEMENT_NAME.equalsIgnoreCase(token)) {
				inStatus = false;
				inProduct = true;
				inCompany = false;
				idx = 0;
				continue;
			} else if (COMPANY_ELEMENT_NAME.equalsIgnoreCase(token)) {
				inStatus = false;
				inProduct = false;
				inCompany = true;
				idx = 0;
				continue;
			}
			idx++;
			if (inStatus) {
				switch(idx) {
					case 1:
						response.setResponseVersion(Integer.valueOf(token));
						break;
					case 2:
						response.setReturnCode(Integer.valueOf(token));
						break;
					case 3:
						response.setRequestType(token);
						break;
					case 4:
						response.setUpc(token);
						break;
				}
			} else if (inProduct) {
				switch(idx) {
					case 1:
						SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH24:mm:ss");
						try {
							product.setLastModified(formatter.parse(token));
						} catch(ParseException e) {
							//Do nothing...
						}
						break;
					case 2:
						product.setEan13(token);
						break;
					case 3:
						product.setUpcA(token);
						break;
					case 4:
						product.setUpcE(token);
						break;
					case 5:
						product.setIsbn(token);
						break;
					case 6:
						product.setName(token);
						break;
					case 7:
						product.setDescription(token);
						break;
					case 8:
						product.setCategoryCode(token);
						break;
					case 9:
						product.setCategoryName(token);
						break;
					case 10:
						product.setCompanyUrl(token);
						break;
					case 11:
						product.setLongDescription(Boolean.valueOf(token));
						break;
					case 12:
						product.setProductImageUrl(token);
						if (autoRetrieveGraphics) {
							product.setProductImage(UpcUtility.retrieveImage(product.getProductImageUrl()));
						}
						break;
					case 13:
						product.setBarcodeUrl(token);
						if (autoRetrieveGraphics) {
							product.setBarcodeImage(UpcUtility.retrieveImage(product.getProductImageUrl()));
						}
						break;
					case 14:
						product.setLocked(Boolean.valueOf(token));
						break;
				}
			} else if (inCompany) {
				switch(idx) {
					case 1:
						company.setName(token);
						break;
					case 2:
						company.setLogoUrl(token);
						if (autoRetrieveGraphics) {
							company.setLogoImage(UpcUtility.retrieveImage(company.getLogoUrl()));
						}
						break;
					case 3:
						company.setUrl(token);
						break;
					case 4:
						company.setAddress(token);
						break;
					case 5:
						company.setTelephone(token);
						break;
					case 6:
						company.setLocked(Boolean.valueOf(token));
						break;
				}
			}
		}
	}

}
