package alrosine.ean.upc.impl;

import alrosine.ean.upc.UpcQueryService;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class UpcQueryServiceFactory {

	private UpcQueryServiceFactory() {
	}

	public static UpcQueryService getUpcQueryService() {
		Properties loadedProperties = getConfigurationProperties();
		Boolean autoRetrieveGraphics = Boolean.valueOf(loadedProperties.getProperty("AutoRetrieveGraphics"));
		return buildUpcQueryService(loadedProperties, autoRetrieveGraphics);
	}

	public static UpcQueryService getUpcQueryService(boolean autoRetrieveGraphics) {
		Properties loadedProperties = getConfigurationProperties();
		return buildUpcQueryService(loadedProperties, autoRetrieveGraphics);
	}

	private static UpcQueryService buildUpcQueryService(Properties loadedProperties, boolean autoRetrieveGraphics) {
		String urlString = loadedProperties.getProperty("Url");
		if (urlString == null) {
			throw new RuntimeException("Url not provided in UpcService.properties file.");
		}
		String apiKeyCode = loadedProperties.getProperty("ApiKeyCode");
		if (apiKeyCode == null) {
			throw new RuntimeException("ApiKeyCode not provided in UpcService.properties file.");
		}
		UpcQueryServiceMode mode = UpcQueryServiceMode.getByMode(loadedProperties.getProperty("Mode"));
		if (mode == null) {
			throw new RuntimeException("Mode not provided in UpcService.properties file.");
		}
		switch(mode) {
			case Csv: {
				UpcQueryParser upcQueryParser = new UpcQueryCsvParser(autoRetrieveGraphics);
				return new UpcQueryServiceImpl(urlString, apiKeyCode, upcQueryParser);
			}
			case Json: {
				UpcQueryParser upcQueryParser = new UpcQueryJsonParser(autoRetrieveGraphics);
				return new UpcQueryServiceImpl(urlString, apiKeyCode, upcQueryParser);
			}
			case Xml: {
				UpcQueryParser upcQueryParser = new UpcQueryXmlParser(autoRetrieveGraphics);
				return new UpcQueryServiceImpl(urlString, apiKeyCode, upcQueryParser);
			}
			default:
				throw new RuntimeException("Unknown UpcQueryServiceMode.");
		}
	}

	private static Properties getConfigurationProperties() {
		Properties loadedProperties = new Properties();
		InputStream in = UpcQueryServiceFactory.class.getResourceAsStream("/UpcService.properties");
		try {
			loadedProperties.load(in);
		} catch(IOException e) {
			throw new RuntimeException("Unable to load UpcService.properties");
		}
		return loadedProperties;
	}
}
