package alrosine.ean.upc.impl;

import alrosine.ean.upc.UpcQueryService;
import alrosine.ean.upc.UpcResponse;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class UpcQueryServiceImpl implements UpcQueryService {

	private String url;
	private String apiKeyCode;
	private String builtUrl;

	private UpcQueryParser upcQueryParser;

	public UpcQueryServiceImpl(String url, String apiKeyCode, UpcQueryParser upcQueryParser) {
		this.setUrl(url);
		this.setApiKeyCode(apiKeyCode);
		this.upcQueryParser = upcQueryParser;
	}

	public UpcResponse getUpcResponse(String upc) {
		if ((upc == null) || (upc.trim().length() == 0)) {
			return null;
		} else
		buildUrl();
		StringBuilder urlBuilder = new StringBuilder(this.builtUrl);
		addUrlProperty(urlBuilder, "find", upc, false);
		return this.upcQueryParser.getUpcResponse(UpcUtility.getUrl(urlBuilder.toString()));
	}

	public InputStream getDataStream(String upc) {
		String url = addRequestedUpc(upc);
		if (url == null) {
			return null;
		} else
		buildUrl();
		StringBuilder urlBuilder = new StringBuilder(this.builtUrl);
		addUrlProperty(urlBuilder, "find", upc, false);
		return UpcUtility.getUrl(urlBuilder.toString());
	}

	public String addRequestedUpc(String requestedUpc) {
		if ((requestedUpc == null) || (requestedUpc.trim().length() == 0)) {
			return null;
		} else
		buildUrl();
		StringBuilder urlBuilder = new StringBuilder(this.builtUrl);
		addUrlProperty(urlBuilder, "find", requestedUpc, false);
		return urlBuilder.toString();
	}

	private Properties loadProperties() {
		Properties loadedProperties = new Properties();
		InputStream in = this.getClass().getResourceAsStream("UpcService.properties");
		try {
			loadedProperties.load(in);
		} catch(IOException e) {
			throw new RuntimeException("Unable to load UpcService.properties");
		}
		return loadedProperties;
	}

	private void buildUrl() {
		if (this.builtUrl == null) {
			StringBuilder urlBuilder = new StringBuilder(this.getUrl());
			addUrlProperty(urlBuilder, "keycode", this.getApiKeyCode(), true);
			addUrlProperty(urlBuilder, "mode", this.getUpcQueryParser().getUpcQueryServiceMode().getValue(), false);
			this.builtUrl = urlBuilder.toString();
		}
	}

	private void addUrlProperty(StringBuilder urlBuilder, String property, String value, boolean firstProperty) {
		if (firstProperty) {
			urlBuilder.append("?");
		} else {
			urlBuilder.append("&");
		}
		urlBuilder.append(property).append("=").append(value);
	}

	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getApiKeyCode() {
		return apiKeyCode;
	}

	public void setApiKeyCode(String apiKeyCode) {
		this.apiKeyCode = apiKeyCode;
	}

	public UpcQueryParser getUpcQueryParser() {
		return upcQueryParser;
	}

	public void setUpcQueryParser(UpcQueryParser upcQueryParser) {
		this.upcQueryParser = upcQueryParser;
	}

}
